(*******************************************************************)
(* dbGen Unit PARSEDB.PAS                                          *)
(* Generated on the 24-10-1993 Sunday, 08:22:20                    *)
(* (c) 1992, Loewy Ron.                                            *)
(*******************************************************************)
unit parsedb;
{$N+,E+,V-,X+,O+,F+} {pxEngine require it this way .. }

interface


(*******************************************************************)
(* Include header definition file                                  *)
(*******************************************************************)
(*******************************************************************)
(* dbGen - OOPS PASCAL generator IML -> Turbo Pascal Code          *)
(* dbGen - (c) 1992,93 Loewy Ron.                                  *)
(* Version 1.0 - File : PARSEDB.HDR                                *)
(*******************************************************************)

(*******************************************************************)
(* Library Units are included here                                 *)
(*******************************************************************)
uses
   tblBase   { table base object definition }
   ,objects  { Collections are defined here }
   ,memTbObj { Memory Table obj  definition }
   ,strings  { NULL ASCIIZ strings unit     }
   ,quickSrt { internal sort used by sProc..}
   ,combSort { external sort used by sProc..}
   ;


(*******************************************************************)
(* Table Definition Section                                        *)
(*******************************************************************)
(*******************************************************************)
(* SYMBOL Table Definition                                         *)
(*******************************************************************)

(*******************************************************************)
(* dataSYMBOL Record Definition                                    *)
(*******************************************************************)
type
   dataSYMBOLPtr = ^ dataSYMBOL;
   dataSYMBOL = record
      wtGeneration : integer;
      NAME : STRING[30];
      VALUE : REAL;
   end; {dataSYMBOL record definition}


(*******************************************************************)
(* TSYMBOLCollection Object Definition                             *)
(*******************************************************************)
type
   PSYMBOLCollection = ^ TSYMBOLCollection;
   TSYMBOLCollection = object(wtCollection)

      function compare(key1, key2 : pointer) : integer; virtual;
      function recUpdate(item : dataSYMBOLPtr) : boolean;
      procedure freeItem(item : pointer); virtual;

end; {TSYMBOLCollection object definition }

type
   SYMBOLPtr = ^ SYMBOL;
   SYMBOL = object(memoryTableObject)

      fieldOffset : array [1 .. 3] of pointer;
      fieldLength : array [1 .. 3] of word;
      tbCollection : PSYMBOLCollection;
      tbCollectIdx : PSYMBOLCollection;
      current : dataSYMBOLPtr;
      prev : dataSYMBOLPtr;
      next : dataSYMBOLPtr;
      tbBuffer : dataSYMBOLPtr;

      constructor init(n : string);
      procedure   initDetails(n : string);
      constructor create(n : string);
      destructor done; virtual;
      procedure translateBuffer(into : dataSYMBOLPtr);
      procedure putToBuffer(from : dataSYMBOLPtr);
      procedure readNextRow; virtual;
      procedure advanceRow; virtual;
      procedure export(nm : string); virtual; { write a fixed block file }
      procedure import(nm : string); virtual; { read in a fixed file }
      procedure modifyCurrent; virtual;

      procedure blankNext; virtual; { for after group processing }
      function bringDirect : boolean;
      procedure writeDirect;
      function deleteDirect : boolean;
      procedure writeRawDirect;

      function bringByKey(p1 : STRING) : boolean;

      procedure writeByKey(p1 : STRING);

      procedure deleteByKey(p1 : STRING);
      function eof : boolean; virtual; { true if at the last record .. }
      procedure appendRecord; virtual;
      procedure empty; virtual;
      procedure updateStatus; virtual;

   end; {SYMBOL object definition}

var
   SYMBOLTbl : SYMBOLPtr;

(*******************************************************************)
(* Global Variable Definition Section                              *)
(*******************************************************************)

(*******************************************************************)
(* Global Procedures Definition Section                            *)
(*******************************************************************)


implementation

(*******************************************************************)
(* Include object procedure definition file                        *)
(*******************************************************************)
(*******************************************************************)
(* User procedures implementation                                  *)
(*******************************************************************)

(*******************************************************************)
(* Here we write the Table object's methods                        *)
(*******************************************************************)

(*******************************************************************)
(* SYMBOL Table methods                                            *)
(*******************************************************************)

(*******************************************************************)
(* TSYMBOLCollection Object methods                                *)
(*******************************************************************)

(*******************************************************************)
(* TSYMBOLCollection.compare                                       *)
(*******************************************************************)
function TSYMBOLCollection.compare;
begin
   compare := 0; { assume equal .. }
   if (dataSYMBOLPtr(key1)^.wtGeneration <
      dataSYMBOLPtr(key2)^.wtGeneration) then begin
      compare := -1;
      exit;
   end else if (dataSYMBOLPtr(key1)^.wtGeneration >
      dataSYMBOLPtr(key2)^.wtGeneration) then begin
      compare := 1;
      exit;
   end;
   if (partialKeyNum <= 1) then
      exit;
   if (dataSYMBOLPtr(key1)^.NAME <
      dataSYMBOLPtr(key2)^.NAME) then begin
      compare := -1;
      exit;
   end else if (dataSYMBOLPtr(key1)^.NAME >
      dataSYMBOLPtr(key2)^.NAME) then begin
      compare := 1;
      exit;
   end;
end; {TSYMBOLCollection.compare}

(*******************************************************************)
(* TSYMBOLCollection.recUpdate                                     *)
(*******************************************************************)
function TSYMBOLCollection.recUpdate;
var
   i : integer;
begin
   if ((not search(keyof(item), i)) or duplicates) then begin
      insert(item);
      recUpdate := false;
   end else begin
      dataSYMBOLPtr(items^[i])^ := item^;
      recUpdate := true; { return true if update }
   end;
end; {TSYMBOLCollection.recUpdate}

procedure TSYMBOLCollection.freeItem;
begin
   if (item <> nil) then begin
      dispose(dataSYMBOLPtr(item));
      item := nil;
   end;
end; {TSYMBOLCollection.freeItem}

constructor SYMBOL.create(n : string);
begin
   init(n);
end; {SYMBOL.create}

(*******************************************************************)
(* SYMBOL.init                                                     *)
(*******************************************************************)
constructor SYMBOL.init;
begin
   tbIndexId := 0; { main index }
   initDetails(n); { this does the actual initialization }
end; {SYMBOL.init}

(*******************************************************************)
(* SYMBOL.initDetails                                              *)
(*******************************************************************)
procedure SYMBOL.initDetails;
begin
   inherited init(n, detail, fixedMedia, 65000); { father will open the table .. }
   new(tbCollection, init(50, 10)); { initialize the collection }
   tbCollectIdx := tbCollection; { by default - just an alias }
   new(tbBuffer); { initialize buffer .. }
   fieldOffset[1] := tbBuffer;
   fieldLength[1] := sizeof(tbBuffer^.wtGeneration);
   fieldOffset[2] := @tbBuffer^.NAME;
   fieldLength[2] := sizeof(tbBuffer^.NAME);
   fieldOffset[3] := @tbBuffer^.VALUE;
   fieldLength[3] := sizeof(tbBuffer^.VALUE);
   new(current);
   new(prev);
   new(next);
end; {SYMBOL.initDetails}


(*******************************************************************)
(* SYMBOL.done                                                     *)
(*******************************************************************)
destructor SYMBOL.done;
begin
   inherited done;
   dispose(current);
   dispose(next);
   dispose(prev);
   dispose(tbBuffer);
   dispose(tbCollection, done);
end; {SYMBOL.done}
(*******************************************************************)
(* SYMBOL.translateBuffer                                          *)
(*******************************************************************)
procedure SYMBOL.translateBuffer;
begin
   into^ := tbBuffer^;
end; {SYMBOL.translateBuffer}

(*******************************************************************)
(* SYMBOL.putToBuffer                                              *)
(*******************************************************************)
procedure SYMBOL.putToBuffer;
begin
   tbBuffer^ := from^;
end; {SYMBOL.putToBuffer}

(*******************************************************************)
(* SYMBOL.advanceRow                                               *)
(*******************************************************************)
procedure SYMBOL.advanceRow;
var
   temp : dataSYMBOLPtr;
begin
   temp := prev;
   prev := current;
   current := next;
   next := temp; { next will be written into .. }
end; {SYMBOL.advanceRow}

(*******************************************************************)
(* SYMBOL.readNextRow                                              *)
(*******************************************************************)
procedure SYMBOL.readNextRow;
begin
   advanceRow;
      inc(collectionCursor);
   if (tbCollection^.count = collectionCursor) then
      if (readToEOT) then
         halt(8)
      else
         readToEOT := true;
   tbBuffer^ := dataSYMBOLPtr(tbCollection^.at(collectionCursor-1))^;
   translateBuffer(next);
end; {SYMBOL.readNextRow}

(*******************************************************************)
(* SYMBOL.export                                                   *)
(*******************************************************************)
procedure SYMBOL.export;
var
   f : file of dataSYMBOL;
begin
   reset;
   assign(f, nm);
   rewrite(f);
   while (not eof) do begin
      tbBuffer^ := dataSYMBOLPtr(tbCollection^.items^[collectionCursor])^;
      inc(collectionCursor);
      translateBuffer(current);
      write(f, current^);
   end; { while .. }
   close(f);
end; {SYMBOL.export}

(*******************************************************************)
(* SYMBOL.import                                                   *)
(*******************************************************************)
procedure SYMBOL.import;
var
   f : file of dataSYMBOL;
   t : dataSYMBOLPtr;
begin
   assign(f, nm);
   system.reset(f);
   while (not system.eof(f)) do begin
      read(f, current^);
      putToBuffer(current);
      new(t);
      t^ := tbBuffer^;
      tbCollection^.atInsert(tbCollection^.count, t);
   end; { while .. }
   close(f);
end; {SYMBOL.import}

procedure SYMBOL.modifyCurrent;
var
   i : integer;
tempo : dataSYMBOLPtr;
begin
   putToBuffer(current);
   i := collectionCursor - 1;
   new(tempo);
   tempo^ := tbBuffer^;
   tbCollection^.atPut(i, tempo);
end; {SYMBOL.modifyCurrent}

procedure SYMBOL.blankNext;
begin
   fillChar(next^, sizeof(dataSYMBOL), #0);
end; {SYMBOL.blankNext}

(*******************************************************************)
(* SYMBOL.bringDirect                                              *)
(*******************************************************************)
function SYMBOL.bringDirect;
var
   rc : integer;
b : boolean;
begin
   putToBuffer(current);
   current^.wtGeneration := wtGenFldBlank;
   b := tbCollection^.Search(current, rc);
   if (b) then
      current^ := dataSYMBOLPtr(tbCollection^.items^[rc])^;
   bringDirect := b;
end; {SYMBOL.bringDirect}

(*******************************************************************)
(* SYMBOL.writeDirect                                              *)
(*******************************************************************)
procedure SYMBOL.writeDirect;
var
   tempTbBuffer : dataSYMBOLPtr;
begin
   current^.wtGeneration := wtGenFldBlank;
   putToBuffer(current);
   new(tempTbBuffer); { current buffer allocated }
   tempTbBuffer^ := tbBuffer^;
   if (tbCollection^.recUpdate(tempTbBuffer)) then
      dispose(tempTbBuffer);
end; {SYMBOL.writeDirect}

(*******************************************************************)
(* SYMBOL.deleteDirect                                             *)
(*******************************************************************)
function SYMBOL.deleteDirect;
var
   rc : integer;
b : boolean;
begin
   current^.wtGeneration := wtGenFldBlank;
   putToBuffer(current);
   b := tbCollection^.Search(current, rc);
   if (b) then
      tbCollection^.atDelete(rc);
   deleteDirect := b;
end; {SYMBOL.deleteDirect}

(*******************************************************************)
(* SYMBOL.writeRawDirect                                           *)
(*******************************************************************)
procedure SYMBOL.writeRawDirect;
var
   tempTbBuffer : dataSYMBOLPtr;
begin
   putToBuffer(current);
   new(tempTbBuffer); { current buffer allocated }
   tempTbBuffer^ := tbBuffer^;
   if (tbCollection^.recUpdate(tempTbBuffer)) then
      dispose(tempTbBuffer);
end; {SYMBOL.writeRawDirect}

function SYMBOL.bringByKey;
begin
   current^.NAME := p1;
   bringByKey := bringDirect;
end; {SYMBOL.bringByKey}

procedure SYMBOL.writeByKey;
begin
   current^.NAME := p1;
   writeDirect;
end; {SYMBOL.writeByKey}

procedure SYMBOL.deleteByKey;
begin
   current^.NAME := p1;
   deleteDirect;
end; {SYMBOL.deleteByKey}

function SYMBOL.eof;
begin
   if (collectionCursor >= tbCollection^.count) then
      eof := true
   else
      eof := false;
end; {SYMBOL.eof}

procedure SYMBOL.appendRecord;
var
   t : dataSYMBOLPtr;
begin
      new(t);
      t^ := tbBuffer^;
      tbCollection^.atInsert(tbCollection^.count, t);
end; {SYMBOL.appendRecord}

procedure SYMBOL.empty;
begin
   tbCollection^.freeAll;
end; {SYMBOL.empty}

procedure SYMBOL.updateStatus;
begin
   tbNumRows := tbCollection^.count;
end; {SYMBOL.updateStatus}

(*******************************************************************)
(* Here we initialize the pascal variables that were given         *)
(* an initial value using the INIT parameter of the VAR statement. *)
(*******************************************************************)

procedure b33_initializeVariables;
begin
end; {b33_initializeVariables}


(*******************************************************************)
(* parsedb Main Loop Driver                                        *)
(*******************************************************************)
begin
end. {parsedb table object unit}
