(******************************************************************************
*                                   tblBase                                   *
* this is the basic table that all tables are descendent from.                *
******************************************************************************)
unit tblBase;

interface

uses
   dbEngBas { database engine basic object is defined here }
{$ifdef windows}
   ,strings
{$endif}
   ;

type 
   tbStorageTypes = (memory, removableMedia, fixedMedia);
   tbType         = (control, detail);

   tableBaseObjectPtr = ^ tableBaseObject;
   tableBaseObject = object

      dbEngine  : databaseEnginePtr; { some things are achived using this }
      tbStorage : tbStorageTypes;
      tbIndexId : integer; { which index to open .. }
{$ifdef WINDOWS}
      tbName    : array[0 .. 38] of char; 
{$else}
      tbName    : string[38]; 
{$endif}
      { table name, will be used as filename if non memory tbStorage }
      tbGroup   : tbType; { control or detail table }
      readToEOT : boolean; { true when we read to end of table .. }
      tbOpen    : boolean;
      lastRead  : longint;
      tbMaxRows : longint; 
      readAllRows : boolean;
      hasIndex  : boolean; { true when table has 2 physical reps. - sorted, 
                             and indexed .. B33 Use }
      tbNumRows : longint; { how much rows are in the table ? }

   (* current, 
      next,
      prev    : dataSOMETHINGPtr; { pointers to the data record the table is built of .. } 
   *)

      constructor init( dbe  : databaseEnginePtr;
                        name : string;
                        typ  : tbType;
                        strg : tbStorageTypes;
                        max  : longInt
                     );
     constructor create(  name : string;
                          typ  : tbType;
                          strg : tbStorageTypes;
                          max  : longint
                       ); { will use pxTblCreate and call init }
     procedure blankNext; virtual;
     procedure bringAllLevels; virtual; { called when a control break occurs }
     procedure modifyCurrent; virtual; { current -> tbRecord -> table }
     procedure writeLevel(level : byte); virtual;
     function readLevel(level : byte) : boolean; virtual;
     procedure import(nm : string); virtual; { read in a fixed file }
     procedure export(nm : string); virtual; { write a fixed block file }
     procedure advanceRow; virtual;
     procedure readNextRow; virtual; { read into next, update current & prev }
     procedure reset; { position at the start of the table .. } virtual;
     function eof : boolean; virtual; { true if at the last record .. }
     procedure activateIndexed(const n : string); virtual;
     function whereInTable : longint; virtual;
     procedure empty; virtual;
     procedure updateStatus; virtual;
     procedure gregorianToDate(m, d, y : integer; var dt); virtual;
     procedure dateToGregorian(const dt; var m, d, y : integer); virtual;
     procedure exportRec(var f : file); virtual;
     procedure importRec(var f : file); virtual;
     function recSize : longint; virtual;

   end; { tableBaseObject table definition }

var
   lastRowFlag : boolean;

var
   genValueSearch : integer;

const
   wtGenFldBlank = $ffffffff; { wtGeneration field blank value .. }

implementation

(******************************************************************************
*                            tableBaseObject.init                             *
******************************************************************************)
constructor tableBaseObject.init;
begin
   tbOpen := false;
   tbStorage := strg;
{$ifdef WINDOWS}
   strPCopy(tbName, name);
{$else}
   tbName := name;
{$endif}
   tbGroup := typ;
   hasIndex := false; { becomes true only after sort on control tables }
   readToEOT := false;
   lastRead := 0;
   readAllRows := true;
   dbEngine := dbe;
end; {tableBaseObject.init}

(******************************************************************************
*                          tableBaseObject.blankNext                          *
******************************************************************************)
procedure tableBaseObject.blankNext;
begin
   (* descendents will over ride *)
end; {tableBaseObject.blankNext}

(******************************************************************************
*                        tableBaseObject.modifyCurrent                        *
******************************************************************************)
procedure tableBaseObject.modifyCurrent;
begin
   (* descendent will perform .. 

      putToBuffer(current);
      go to prev record .. 
      updateRecord; .. 
      goto next record .. 

   *)
end; {tableBaseObject.modifyCurrent}

(******************************************************************************
*                       tableBaseObject.bringAllLevels                        *
******************************************************************************)
procedure tableBaseObject.bringAllLevels;
begin
   (* when a beforeGroup, or afterGroup procedure is to be executed, 
      call this procedure to bring all levels to memory .. *)
end; {tableBaseObject.bringAllLevels}

(******************************************************************************
*                         tableBaseObject.writeLevel                          *
******************************************************************************)
procedure tableBaseObject.writeLevel;
begin
   (* let descendents override - write level specified from current, 
      to control table
   *)
end; {tableBaseObject.writeLevel}

(******************************************************************************
*                          tableBaseObject.readLevel                          *
******************************************************************************)
function tableBaseObject.readLevel;
begin
   (* let descendents override - read level specified in current, 
      from control table
      return TRUE if record found, false otherwise
   *)
end; {tableBaseObject.readLevel}

(******************************************************************************
*                           tableBaseObject.import                            *
******************************************************************************)
procedure tableBaseObject.import;
begin
(*
   should look like :

var
   f : file of dataSomething;

   assign(f, nm);
   system.reset(f);
   while (not eof(f)) do begin
      read(f, current^);
      putToBuffer(current);
      pxRecAppend(tbHandle, tbRecord);
   end; { while }
*)
end; {tableBaseObject.import}

(******************************************************************************
*                           tableBaseObject.export                            *
******************************************************************************)
procedure tableBaseObject.export;
begin
(* shold look like : 
var
   f : file of dataSomething;

   reset;
   assign(f, nm);
   rewrite(f);
   while (not eof) do begin
      readRecord;
      translateBuffer(current);
      write(f, current^);
   end; { while .. }
   close(f);
*)
end; {tableBaseObject.export}

(******************************************************************************
*                         tableBaseObject.advanceRow                          *
******************************************************************************)
procedure tableBaseObject.advanceRow;
begin
   (* what should be written is .. 
      prev := current;
      current := next;
   *)
end; {tableBaseObject.advanceRow}

(******************************************************************************
*                           tableBaseObject.create                            *
******************************************************************************)
constructor tableBaseObject.create;
begin
   { descendents will override -to perform pxTblCreate and pxKeyAdd }
   { descendent MUST call init afterwards ! }
(*   init(name, typ, strg, max); *)
end; {tableBaseObject.create}

(******************************************************************************
*                         tableBaseObject.readNextRow                         *
******************************************************************************)
procedure tableBaseObject.readNextRow;
begin
   (* descendents will over ride *)
end; {tableBaseObject.readNextRow}

(******************************************************************************
*                            tableBaseObject.reset                            *
******************************************************************************)
procedure tableBaseObject.reset;
begin
   (* descendents will over ride *)
   lastRead := 0;
   readToEOT := false;
end; {tableBaseObject.reset}

(******************************************************************************
*                             tableBaseObject.eof                             *
******************************************************************************)
function tableBaseObject.eof;
begin
   eof := true;
   (* descendents will over ride *)
end; {tableBaseObject.eof}

(******************************************************************************
*                        tableBaseObject.whereInTable                         *
******************************************************************************)
function tableBaseObject.whereInTable;
begin
end; {tableBaseObject.whereInTable}

(******************************************************************************
*                       tableBaseObject.activateIndexed                       *
******************************************************************************)
procedure tableBaseObject.activateIndexed;
begin
   hasIndex := true; 
   { tbIndexed holds the handle to the indexed table, while tbHandle will hold
     the handle to the sorted - non indexed table }
end; {tableBaseObject.activateIndexed}

(******************************************************************************
*                            tableBaseObject.empty                            *
******************************************************************************)
procedure tableBaseObject.empty;
begin
end; {tableBaseObject.empty}

(******************************************************************************
*                        tableBaseObject.updateStatus                         *
******************************************************************************)
procedure tableBaseObject.updateStatus;
begin
end; {tableBaseObject.updateStatus}

(******************************************************************************
*                       tableBaseObject.gregorianToDate                       *
******************************************************************************)
procedure tableBaseObject.gregorianToDate;
begin
   if (dbEngine <> nil) then
      gregorianToDate(m, d, y, dt);
end; {tableBaseObject.gregorianToDate}

(******************************************************************************
*                       tableBaseObject.dateToGregorian                       *
******************************************************************************)
procedure tableBaseObject.dateToGregorian;
begin
   if (dbEngine <> nil) then
      dbEngine^.dateToGregorian(dt, m, d, y);
end; {tableBaseObject.dateToGregorian}

(******************************************************************************
*                          tableBaseObject.importRec                          *
******************************************************************************)
procedure tableBaseObject.importRec;
begin
end; { tableBaseObject.importRec }

(******************************************************************************
*                          tableBaseObject.exportRec                          *
******************************************************************************)
procedure tableBaseObject.exportRec;
begin
   { descendents will override }
end; { tableBaseObject.exportRec }

(******************************************************************************
*                           tableBaseObject.recSize                           *
******************************************************************************)
function tableBaseObject.recSize;
begin
end; { tableBaseObject.recSize }

end.
